/*
* This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/bonsai
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Norman Köster, Torben Töniges,
* 				Annika Rothert, Alexander Baumann, Marcel Dulisch, Maikel Linke,
* 				Marian Pohling, Lukas Twardon
*/
package de.unibi.airobots.bonsai.examples;

import java.io.IOException;
import java.net.URISyntaxException;

import nu.xom.Attribute;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;

import de.unibi.airobots.bonsai.actuators.EnterArenaAndLocalizeActuator;
import de.unibi.airobots.bonsai.core.BonsaiManager;

public class EnterArenaTest {
    
    public static void main(String[] args) {
        
        Logger logger = Logger.getLogger(EnterArenaTest.class);
        
        BasicConfigurator.configure();
        
        try{
            BonsaiManager.getInstance().configure(
                    EnterArenaTest.class.getResource("enterarenatest.xml").toURI());
        }
        catch(URISyntaxException e){
            logger.error("Configuration file not found.", e);
        }
        
        EnterArenaAndLocalizeActuator actuator = BonsaiManager.getInstance().createActuator(
                "EnterArenaAndLocalizeActuator", EnterArenaAndLocalizeActuator.class);
        
        try {
            // Move 2 meters, then localize.
            // Prior knowledge: The robot might be at about x=2, y=0 on the map (TODO: Annotation on the map).
            // But we're not sure, so we say div = 2 meters.
            actuator.enterArenaAndLocalize(2.0, 2.0, 0.0, 2.0);
            logger.info("Enter arena and localize...");
            actuator.waitUntilLocalizationFinished();
            logger.info("Localization finished!");
        } catch (IOException e) {
           logger.error(e);
        }
        
    }

}
