package de.unibi.airobots.bonsai.examples;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import javax.swing.JFrame;
import javax.swing.JPanel;

import org.apache.log4j.BasicConfigurator;

import de.unibi.airobots.bonsai.core.BonsaiManager;
import de.unibi.airobots.bonsai.core.Sensor;
import de.unibi.airobots.bonsai.util.slam.SlamTools;
import de.unibi.airobots.bonsai.util.slam.gui.MapPanel;
import de.unibi.airobots.btl.data.BinarySlamMap;

import net.sf.xcf.InitializeException;
import net.sf.xcf.Subscriber;
import net.sf.xcf.XcfManager;
import net.sf.xcf.event.PublishEvent;
import net.sf.xcf.event.XcfEvent;
import net.sf.xcf.examples.SystemEventPrinter;
import net.sf.xcf.naming.NameNotFoundException;
import net.sf.xcf.util.SynchronizedQueue;

public class FusedMapContainer extends JFrame {

    private SynchronizedQueue<PublishEvent> queue;
    private GridBagLayout gbl;
    private Sensor<BinarySlamMap> mapSensor;
    private JPanel fmw;
    private Sensor<BinarySlamMap> laserSensor;
    private JPanel lmw;
    private JPanel smw;
    
    /** Used for periodically updating the map data. */
    private final ScheduledExecutorService executor =
            Executors.newSingleThreadScheduledExecutor();

    public FusedMapContainer() {
        super("MultiSensorFusion");
        
        BasicConfigurator.configure();
        
        // Set up Fused Map visualization.
        gbl = new GridBagLayout();
        this.setLayout(gbl);
        
        
        mapSensor = BonsaiManager.getInstance().createSensor("FusedMapSensor",
                BinarySlamMap.class);
        fmw = new JPanel();
        fmw.setPreferredSize(new Dimension(320, 320));
        
        /*
        laserSensor = BonsaiManager.getInstance().createSensor("LaserMapSensor",
                BinarySlamMap.class);
        lmw = new JPanel();
        lmw.setPreferredSize(new Dimension(160, 160));
        
        mapSensor = BonsaiManager.getInstance().createSensor("SlamSensor",
                BinarySlamMap.class);
        smw = new JPanel();
        smw.setPreferredSize(new Dimension(160, 160));
        */
        
        this.addComponent(this.getContentPane(), gbl, fmw, 0, 0, 1, 2, 1, 1);
        //this.addComponent(this.getContentPane(), gbl, smw, 1, 0, 1, 1, 1, 1);
        //this.addComponent(this.getContentPane(), gbl, lmw, 1, 1, 1, 1, 1, 1);
        this.pack();
        this.setVisible(true);
        
        executor.scheduleWithFixedDelay(new Runnable() {
            @Override
            public void run() {
                paintWindows();
            }
        }, 200, 100, TimeUnit.MILLISECONDS);
    }

    /**
     * A useful function to add all components into the GridBagLayout.
     */
    private void addComponent(Container cont, GridBagLayout gbl, Component c,
            int x, int y, int width, int height, double weightx, double weighty) {
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.BOTH;
        gbc.gridx = x;
        gbc.gridy = y;
        gbc.gridwidth = width;
        gbc.gridheight = height;
        gbc.weightx = weightx;
        gbc.weighty = weighty;
        gbl.setConstraints(c, gbc);
        cont.add(c);
    }
    
    public void paint(Graphics g) {
        this.paintWindows();
    }
    
    private void paintWindows() {
        try {
            BinarySlamMap fusedMap = mapSensor.readLast(0);
            BufferedImage fusedMapImage = SlamTools.slamMapToImage(fusedMap);
            Graphics2D g = (Graphics2D) fmw.getGraphics();
            AffineTransform imageTransform =
                    new AffineTransform(fmw.getWidth()
                            / (double) fusedMap.getWidth(), 0, 0,
                            fmw.getHeight() / (double) fusedMap.getHeight(), 0,
                            0);
            g.drawImage(fusedMapImage, imageTransform, this);
            
            /*
            BinarySlamMap laserMap = mapSensor.readLast(0);
            BufferedImage laserMapImage = SlamTools.slamMapToImage(fusedMap);
            g = (Graphics2D) fmw.getGraphics();
            imageTransform =
                    new AffineTransform(fmw.getWidth()
                            / (double) fusedMap.getWidth(), 0, 0,
                            fmw.getHeight() / (double) fusedMap.getHeight(), 0,
                            0);
            g.drawImage(fusedMapImage, imageTransform, this);
            */
            
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
