/*
* This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/bonsai
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Norman Köster, Torben Töniges,
* 				Annika Rothert, Alexander Baumann, Marcel Dulisch, Maikel Linke,
* 				Marian Pohling, Lukas Twardon
*/
package de.unibi.airobots.bonsai.examples;

import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.BasicConfigurator;

import net.sf.xcf.MethodNotFoundException;
import net.sf.xcf.RemoteServer;
import net.sf.xcf.UserException;
import net.sf.xcf.XcfManager;
import net.sf.xcf.transport.XOPData;
import net.sf.xcf.xml.SyntaxException;
import de.unibi.airobots.bonsai.core.Actuator;
import de.unibi.airobots.bonsai.xcf.RemoteServerReconnectable;
import de.unibi.airobots.bonsai.xcf.RemoteServerReconnectableManager;

public class TestServerCaller {

    private static class TestActuator implements Actuator,
            RemoteServerReconnectable {

        private RemoteServer server = null;

        /**
         * {@inheritDoc}
         */
        @Override
        public Map<String, ErrorHandlingPolicy> getRemoteServerNames() {
            Map<String, ErrorHandlingPolicy> servers =
                    new HashMap<String, ErrorHandlingPolicy>();
            servers.put(TestServer.SERVER_NAME,
                    ErrorHandlingPolicy.SELF_HEALING);
            return servers;
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public void resetRemoteServer(RemoteServer remoteServer) {
            this.server = remoteServer;
        }

        public void callMethod() {

            if (server == null) {
                throw new IllegalStateException();
            }

            try {
                System.out.println("Calling method");
                XOPData result =
                        server.callMethod(TestServer.METHOD_NAME, XOPData
                                .fromString("<callIt/>"));
                System.out.println("result received: " + result);
            } catch (MethodNotFoundException e) {
                e.printStackTrace();
            } catch (UserException e) {
                e.printStackTrace();
            } catch (SyntaxException e) {
                e.printStackTrace();
            }

        }

    }

    public static void main(String[] args) {

        try {

            BasicConfigurator.configure();

            XcfManager xcfManager = XcfManager.createXcfManager();
            RemoteServerReconnectableManager manager =
                    new RemoteServerReconnectableManager(xcfManager);
            manager.initializeXcfType(TestServer.SERVER_NAME);

            TestActuator actuator = new TestActuator();
            manager.initializeAndManage(actuator);

            // wait until the actuator is probably initialized with the
            // server...
            while (true) {

                actuator.callMethod();
                System.out.println("Waiting 2 seconds");
                Thread.sleep(2000);

            }

        } catch (Exception e) {
            e.printStackTrace();
            System.exit(1);
        }

    }

}
