/*
* This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/bonsai
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Norman Köster, Torben Töniges,
* 				Annika Rothert, Alexander Baumann, Marcel Dulisch, Maikel Linke,
* 				Marian Pohling, Lukas Twardon
*/
package de.unibi.airobots.bonsai.skills;

import java.io.IOException;
import java.lang.reflect.Constructor;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;

import org.apache.commons.scxml.io.SCXMLParser;
import org.apache.commons.scxml.model.ModelException;
import org.apache.commons.scxml.model.SCXML;
import org.apache.commons.scxml.model.State;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import de.unibi.airobots.bonsai.core.Actuator;
import de.unibi.airobots.bonsai.core.Sensor;
import de.unibi.airobots.bonsai.skills.state.AbstractState;

/**
 * @author nkoester
 */
public class SCXMLDecoder {

    /**
     * 
     */
    private SCXML scxml;

    private SCXMLValidator scxmlValidator;
    
    /**
     * 
     */
    private LinkedList<AbstractState> stateChain;

    /**
     * 
     */
    private LinkedList<? extends Sensor<?>> sensorList;

    /**
     * 
     */
    private LinkedList<? extends Actuator> actuatorList;

    /**
     * 
     */
    public SCXMLDecoder() {
        scxmlValidator = new SCXMLValidator();
        stateChain = new LinkedList<AbstractState>();
        sensorList = new LinkedList<Sensor<?>>();
        actuatorList = new LinkedList<Actuator>();
    }

    
    public SCXML parseAndValidateSCXML2(String pathToSCXML) {
        parseAndValidateSCXML(pathToSCXML);
        return getScxml();
    }
    
    /**
     * @param pathToSCXML
     * @return
     * 
     *         TODO: handle errors correctly ...
     * 
     * 
     */
    public boolean parseAndValidateSCXML(String pathToSCXML) {

        // parsing SCXML
        // TODO: lookup if there is any alternative for error handling in the
        // parsing process ...
        ErrorHandler errorHandler = new ErrorHandler() {

            @Override
            public void warning(SAXParseException exception)
                    throws SAXException {
                System.out.println("warning!");
            }

            @Override
            public void fatalError(SAXParseException exception)
                    throws SAXException {
                System.out.println("fatal!");
            }

            @Override
            public void error(SAXParseException exception) throws SAXException {
                System.out.println("err!");
            }
        };

        // the actual parsing of the SCXML
        try {
            URL scxmlURL = new URL(pathToSCXML);
            scxml = SCXMLParser.parse(scxmlURL, errorHandler);
        } catch (MalformedURLException e1) {
            System.out.println("Exception while digesting SCXML: "
                    + e1.toString());
            System.exit(-1);
        } catch (IOException e2) {
            System.out.println("Exception while digesting SCXML: "
                    + e2.toString());
            System.exit(-1);
        } catch (SAXException e3) {
            System.out.println("Exception while digesting SCXML: "
                    + e3.toString());
            System.exit(-1);
        } catch (ModelException e4) {
            System.out.println("Exception while digesting SCXML: "
                    + e4.toString());
            System.exit(-1);
        }

        System.out.println("SCXML digesting successful!");

        // checking if the states defined in the given SCXML are valid
        return scxmlValidator.validateSCXML(this.scxml);
    }

    /**
     * Validates the list of sensors and actuators specified for the skill.
     * TODO: implement ...? or use the factory for this ...
     * 
     * @return
     */
    private boolean validateActuatorSensorList() {
        // validate actuator and sensor lists here
        return true;
    }



    /** Returns the 
     * @return
     */
    public LinkedList<? extends AbstractState> getStateChain() {
        return stateChain;
    }

    /**
     * @return
     */
    public LinkedList<? extends Sensor<?>> getSensorList() {
        return sensorList;
    }

    /**
     * @return
     */
    public LinkedList<? extends Actuator> getActuatorList() {
        return actuatorList;
    }

    
    public SCXML getScxml() {
        return scxml;
    }

    
    public void setScxml(SCXML scxml) {
        this.scxml = scxml;
    }
    
}
