/*
 * This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
 *
 * Copyright(c) Frederic Siepmann
 * http://opensource.cit-ec.de/projects/bonsai
 *
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 *
 * Contributors: Leon Ziegler, Johannes Wienke, Norman Köster, Torben Töniges,
 * 				Annika Rothert, Alexander Baumann, Marcel Dulisch, Maikel Linke,
 * 				Marian Pohling, Lukas Twardon
 */
package de.unibi.airobots.bonsai.skills.state;

import java.io.IOException;

import org.apache.log4j.Logger;

import de.unibi.airobots.bonsai.actuators.NavigationActuator;
import de.unibi.airobots.bonsai.actuators.SpeechActuator;
import de.unibi.airobots.bonsai.core.BonsaiManager;
import de.unibi.airobots.bonsai.core.Sensor;
import de.unibi.airobots.bonsai.skills.SkillStateMachine;
import de.unibi.airobots.bonsai.util.helper.Movements;
import de.unibi.airobots.btl.data.GlobalPlannerStateChange;
import de.unibi.airobots.btl.data.LaserData;
import de.unibi.airobots.btl.data.NavigationGoalData;
import de.unibi.airobots.btl.data.PositionData;
import de.unibi.airobots.btl.data.SpeedData;

/**
 * Implementation of a simple follow person state.
 * 
 * @author nkoester
 */
public class DriveToRegistration extends AbstractState {

    private static final String GO_TO_REGISTRATION_DESK =
            "Ok, i will go to the registration desk.";
    private Logger logger = Logger.getLogger(this.getClass());

    
    private Sensor<GlobalPlannerStateChange> gplanChangeSensor;
    private NavigationActuator navActuator;    
    private Sensor<PositionData> positionSensor;
    private Sensor<SpeedData> speedSensor;
    
    
    public DriveToRegistration(SkillStateMachine aStateMachine) {
        super(aStateMachine);
    }

    @Override
    public void getRequiredSensorsAndActuators() {
        navActuator = BonsaiManager.getInstance().createActuator(
                "NavigationActuator", NavigationActuator.class);
        positionSensor = BonsaiManager.getInstance().createSensor(
                "PositionSensor", PositionData.class);
        gplanChangeSensor = BonsaiManager.getInstance().createSensor(
                "GlobalPlannerStateChangeSensor",
                GlobalPlannerStateChange.class);
        speedSensor = BonsaiManager.getInstance().createSensor(
                "SpeedSensor", SpeedData.class);

    }

    @Override
    public boolean init() {
        System.out.println("\t" + this.getClass().getSimpleName().toString()
                + " init");
        return true;
    }

    @Override
    public boolean execute() {

        System.out.println("\t" + this.getClass().getSimpleName().toString()
                + " execute");
        System.out.println("setting goal");
        NavigationGoalData target =
                new NavigationGoalData(new PositionData(3, 3, 3,
                        System.currentTimeMillis()));
        target.setYawTolerance(Math.PI / 5);
        System.out.println("driving there now");

        try {
            if (Movements.driveToPosition(navActuator,
                    positionSensor, speedSensor,
                    gplanChangeSensor, target)) {
                System.out.println("tureee");
                return true;
            } else {
                System.out.println("some other error");
                return false;
            }
        } catch (IOException e) {
            e.printStackTrace();
            System.out.println("omg: " + e.getMessage());
            return false;
        }
    }

    @Override
    public boolean end() {

        System.out.println("\t" + this.getClass().getSimpleName().toString()
                + " end");

        return true;
    }

}
