/*
 * This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
 *
 * Copyright(c) Frederic Siepmann
 * http://opensource.cit-ec.de/projects/bonsai
 *
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 *
 * Contributors: Leon Ziegler, Johannes Wienke, Norman Köster, Torben Töniges,
 * 				Annika Rothert, Alexander Baumann, Marcel Dulisch, Maikel Linke,
 * 				Marian Pohling, Lukas Twardon
 */
package de.unibi.airobots.bonsai.skills.state;

import java.io.IOException;

import org.apache.log4j.Logger;

import de.unibi.airobots.bonsai.actuators.EnterArenaAndLocalizeActuator;
import de.unibi.airobots.bonsai.actuators.SpeechActuator;
import de.unibi.airobots.bonsai.core.BonsaiManager;
import de.unibi.airobots.bonsai.skills.SkillStateMachine;
import de.unibi.airobots.btl.data.LaserData;
import de.unibi.airobots.btl.data.SpeedData;

/**
 * Implementation of a simple follow person state.
 * 
 * @author nkoester
 */
public class Localise extends AbstractState {

    private Logger logger = Logger.getLogger(this.getClass());
    private static final double STEP_FORWARD = 2.0;

    
    private EnterArenaAndLocalizeActuator enterArenaActuator;
    private SpeechActuator speechActuator;
    
    public Localise(SkillStateMachine aStateMachine) {
        super(aStateMachine);
    }
    
    public void getRequiredSensorsAndActuators() {
        this.speechActuator =
                BonsaiManager.getInstance().createActuator("SpeechActuator",
                        SpeechActuator.class);
        enterArenaActuator = BonsaiManager.getInstance().createActuator(
                "EnterArenaAndLocalizeActuator",
                EnterArenaAndLocalizeActuator.class);
    }

    @Override
    public boolean init() {
        System.out.println("\t" + this.getClass().getSimpleName().toString()
                + " init");

        try {
            speechActuator.sayAccentuated("Ok. First let me check where I am.");
        } catch (IOException e) {
            e.printStackTrace();
        }

        return true;
    }

    @Override
    public boolean execute() {

        System.out.println("\t" + this.getClass().getSimpleName().toString()
                + " execute");

        try {

            String door = "door1";

            double x = 1.5;
            double y = 1.5;

            logger.info("goal x: " + x + " y: " + y);

            enterArenaActuator
                    .enterArenaAndLocalize(STEP_FORWARD, x, y, 2);
            logger.debug("enter arena and localize");
            enterArenaActuator.waitUntilLocalizationFinished();
            logger.debug("localization finished");
        } catch (Exception e) {
            logger.error(e);
        }

        return true;
    }

    @Override
    public boolean end() {

        System.out.println("\t" + this.getClass().getSimpleName().toString()
                + " end");

        return true;
    }
}
