package de.unibi.airobots.bonsai.util.arm;

import java.io.FileInputStream;
import java.io.IOException;

import org.apache.log4j.Logger;

import de.unibi.airobots.btl.common.Point3D;
import de.unibi.airobots.btl.units.LengthUnit;

import Jama.Matrix;

public class CameraArmTransformation {

    private Matrix transformationMatrix;
    private static final String DEFAULT_MATRIX_FILE =
        "/vol/robocup/trunk/etc/armcalibration/transformationMatrixOutput";

    public CameraArmTransformation(String matrixFile) {
        this.transformationMatrix = this.parseMatrixFile(matrixFile);
    }
    public CameraArmTransformation() {
        this(DEFAULT_MATRIX_FILE);
    }

    public Matrix transform(Matrix kinectCoords) {
        if (this.transformationMatrix == null) {
            System.err.println("Transformation Matrix not present!");
            return null;
        }
        return this.transformationMatrix.times(kinectCoords);
    }

    public Point3D transform(Point3D p) {
        /*Matrix in = new Matrix(4, 1);
        in.set(0, 0, p.getX(LengthUnit.MILLIMETER));
        in.set(1, 0, p.getY(LengthUnit.MILLIMETER));
        in.set(2, 0, p.getZ(LengthUnit.MILLIMETER));
        in.set(3, 0, 1.0);*/
        double[][] inSrc = {{p.getX(LengthUnit.MILLIMETER)},
                {p.getY(LengthUnit.MILLIMETER)},
                {p.getZ(LengthUnit.MILLIMETER)},
                {1.0}};
        Matrix in = new Matrix(inSrc);
        Matrix out = this.transform(in);
        return new Point3D(out.get(0, 0), out.get(1, 0), out.get(2, 0),
                LengthUnit.MILLIMETER);
    }

    public void setTransformationMatrix(Matrix t) {
        this.transformationMatrix = t;
    }

    /**
     * Read a transformation matrix file and build a matrix.
     * 
     * @param filename
     *            Path to the transformation matrix file.
     * @return the matrix.
     */
    private Matrix parseMatrixFile(String filename) {
        try {
            FileInputStream fs = new FileInputStream(filename);
            int ch;
            StringBuffer strContent = new StringBuffer("");
            while ((ch = fs.read()) != -1)
                strContent.append((char) ch);
            fs.close();
            String input = strContent.toString();
            String rows[] = input.split("\n");
            Matrix M = new Matrix(4, 4);
            for (int i = 0; i < rows.length; i++) {
                String[] columns = rows[i].split(" ");
                for (int j = 0; j < rows.length; j++) {
                    M.set(i, j, Double.parseDouble(columns[j]));
                }
            }
            return M;
        } catch (IOException e) {
            Logger.getRootLogger().error(
                    "Error reading matrix file '" + filename + "'.");
            return null;
        } catch (IndexOutOfBoundsException e) {
            Logger.getRootLogger().error(
                    "Error parsing matrix file '" + filename
                            + "'. Make sure the matrix is 4x4!");
            return null;
        }
    }

    public void setTransformationMatrix(String filename) {
        Matrix m = this.parseMatrixFile(filename);
        this.setTransformationMatrix(m);
    }
}
