/*
* This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/bonsai
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Norman Köster, Torben Töniges,
* 				Annika Rothert, Alexander Baumann, Marcel Dulisch, Maikel Linke,
* 				Marian Pohling, Lukas Twardon
*/
package de.unibi.airobots.bonsai.util.helper;

import de.unibi.airobots.btl.data.PersonData;
import de.unibi.airobots.btl.data.PositionData;

/**
 * This is a collection of information concerning one person in the scenario.
 * 
 * @author lziegler
 */
public class PersonDescription {

	/**
	 * Used Names.
	 */
	public static String[] NAMES = { "Guybrush", "Elaine", "Largo", "Kate",
			"Carla" };

	/**
	 * A static counter for the generation of new IDs when a new
	 * {@link PersonDescription} is created.
	 */
	private static int idCounter = 0; // start with 1 because 0 is fix for
										// pre-learned person.

	// private static int idCounter = 0;

	/**
	 * Returns the name for the person with the given id.
	 * 
	 * @param id
	 *            id to search the name for
	 * @return name
	 */
	private static String getNameById(final int id) {

		if (id == 0) {
			return "A";
		} else if (id == 1) {
			return "B";
		} else {
			
			int idWithOffset = id - 2;

			String name = NAMES[idWithOffset % NAMES.length];

			if (idWithOffset >= NAMES.length) {
				name += " ";
				name += Math.floor(idWithOffset / NAMES.length) + 1;
			}

			return name;
		}
	}

	private String name;
	private PersonData trackingPerson;
	private PositionData lastPosition = new PositionData(0, 0, 0, 0);
	private int id;

	public PersonDescription() {
		this.id = idCounter++;
		this.name = getNameById(id);
	}

	public PersonDescription(PersonData person, PositionData position) {
		this.id = idCounter++;
		this.name = getNameById(id);
		this.trackingPerson = person;
		this.lastPosition = position;
	}

	public PersonDescription(String name, PersonData person, PositionData position) {
		this.id = idCounter++;
		this.name = name;
		this.trackingPerson = person;
		this.lastPosition = position;
	}
	
	public PersonDescription(String name, PersonData person, PositionData position, int id) {
		this.id = id;
		this.name = name;
		this.trackingPerson = person;
		this.lastPosition = position;
	}

	public PersonDescription(String string, int id2) {
		this.id = id2;
		this.name = string;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public PersonData getTrackingPerson() {
		return trackingPerson;
	}

	public void setTrackingPerson(PersonData trackingPerson) {
		this.trackingPerson = trackingPerson;
	}

	public PositionData getLastPosition() {
		return lastPosition;
	}

	public void setLastPosition(PositionData lastPosition) {
		this.lastPosition = lastPosition;
	}

	public int getId() {
		return id;
	}
	
	public void setId(int id) {
		this.id = id;
	}

}
