/*
* This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/bonsai
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Norman Köster, Torben Töniges,
* 				Annika Rothert, Alexander Baumann, Marcel Dulisch, Maikel Linke,
* 				Marian Pohling, Lukas Twardon
*/
package de.unibi.airobots.bonsai.util.helper.annotation;

import java.util.LinkedList;
import java.util.List;

import de.unibi.airobots.btl.data.Type;

/**
 * 
 * @author jhelbach
 */
public class MapArea extends Type {

	public static enum MapAreaType {

		NOGO, ROOM, INTEREST, PERSON, DISTURB, OBJECTS, NO_TYPE;

	}

	private MapAreaType type;

	private String arenaName;

	private LinkedList<MapRegion> regions;

	public MapArea(MapArea.MapAreaType type) {
		this.arenaName = "";
		this.regions = new LinkedList<MapRegion>();
		this.type = type;
	}

	public MapArea(MapAreaType type, String arenaName,
			LinkedList<MapRegion> regions) {
		this.type = type;
		this.arenaName = arenaName;
		this.regions = regions;
	}

	public void addRegion(MapRegion region) {

		String label = createNewLabel(region);

		region.setLabel(label);
		regions.add(region);
	}

	private boolean checkRegionLabel(String region) {

		for (MapRegion r : regions) {
			if (r.getLabel().equals(region)) {
				return false;
			}
		}
		return true;
	}

	private String createNewLabel(MapRegion region) {

		int i = 0;
		String label = region.getLabel();

		while (!checkRegionLabel(label)) {
			i++;
			label = label.concat(String.valueOf(i));
		}

		return label;
	}

	public MapArea.MapAreaType getType() {
		return type;
	}

	public void setType(MapArea.MapAreaType type) {
		this.type = type;
	}

	public String getArenaName() {
		return arenaName;
	}

	public void setArenaName(String arenaName) {
		this.arenaName = arenaName;
	}

	public LinkedList<MapRegion> getRegions() {
		return regions;
	}

	public void setRegions(LinkedList<MapRegion> regions) {
		this.regions = regions;
	}

	public void removeRegion(String regionLabel) {

		LinkedList<MapRegion> copy = new LinkedList<MapRegion>(regions);

		for (MapRegion r : copy) {

			if (r.getLabel().equals(regionLabel)) {
				regions.remove(r);
			}

		}

	}

	public MapRegion getRegionByLabel(String regionLabel) {
		for (MapRegion r : regions) {

			if (r.getLabel().equals(regionLabel)) {
				return r;
			}

		}
		return null;

	}

	public List<MapRegion> getRegionByGroup(String group) {

		LinkedList<MapRegion> r = new LinkedList<MapRegion>();

		for (MapRegion mr : regions) {
			if (mr.getGroup().equals(group)) {
				r.add(mr);
			}
		}

		return r;

	}

}
