/*
* This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/bonsai
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Norman Köster, Torben Töniges,
* 				Annika Rothert, Alexander Baumann, Marcel Dulisch, Maikel Linke,
* 				Marian Pohling, Lukas Twardon
*/
package de.unibi.airobots.bonsai.util.slam;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;

import de.unibi.airobots.bonsai.util.helper.annotation.MapArea;
import de.unibi.airobots.bonsai.util.helper.annotation.MapPoint;
import de.unibi.airobots.bonsai.util.helper.annotation.MapRegion;
import de.unibi.airobots.bonsai.util.xml.XMLEncoderUtil;

public class AnnotationDataTreePanel extends JPanel implements ActionListener {

    public static final String DRIVE2 = "drive";

    public static final String EDIT2 = "edit";

    public static final String REMOVE2 = "remove";

    /**
     * 
     */
    private static final long serialVersionUID = 2675588142637432104L;

    private JTree annotationTree;
    private DefaultMutableTreeNode rootNode;
    private JScrollPane scrollPane;

    private List<AnnotationChangeListener> listeners =
            new ArrayList<AnnotationChangeListener>();

    public AnnotationDataTreePanel() {
        this.setLayout(new BorderLayout());
        scrollPane = new JScrollPane();
        this.add(scrollPane, BorderLayout.CENTER);

        rootNode = new DefaultMutableTreeNode("Annotation");
        annotationTree = new JTree(rootNode);

        scrollPane.setViewportView(annotationTree);

        JPanel buttonPanel = new JPanel(new GridLayout(1, 2));

        JButton remove = new JButton(REMOVE2);
        remove.addActionListener(this);
        buttonPanel.add(remove);

        JButton edit = new JButton(EDIT2);
        edit.addActionListener(this);
        buttonPanel.add(edit);
        
        JButton drive = new JButton(DRIVE2);
        drive.addActionListener(this);
        buttonPanel.add(drive);

        this.add(buttonPanel, BorderLayout.SOUTH);

    }
    
    public void addAnnotation(MapArea area) {

        DefaultMutableTreeNode areaNode =
                new DefaultMutableTreeNode(area.getType());

        for (MapRegion region : area.getRegions()) {
            DefaultMutableTreeNode regionNode =
                    new DefaultMutableTreeNode(region.getLabel());

            if (region.getViewPoint() != null) {
                DefaultMutableTreeNode pointNode =
                    new DefaultMutableTreeNode(
                            XMLEncoderUtil.VIEW_POINT_ELEMENTNAME + " "
                                    + region.getViewPoint().getX() + " " + region.getViewPoint().getY());
            regionNode.add(pointNode);
            }
            
            for (MapPoint point : region.getPoints()) {
                DefaultMutableTreeNode pointNode =
                        new DefaultMutableTreeNode(
                                XMLEncoderUtil.POINT_ATTRNAME + " "
                                        + point.getX() + " " + point.getY());
                regionNode.add(pointNode);
            }

            areaNode.add(regionNode);
        }

        rootNode.add(areaNode);

        annotationTree = new JTree(rootNode);

        scrollPane.setViewportView(annotationTree);

    }

    public void addListener(AnnotationChangeListener listener) {
        listeners.add(listener);
    }

    public void removeListener(AnnotationChangeListener listener) {
        listeners.remove(listener);
    }

    public void notifyAnnotationListeners(String event, DefaultMutableTreeNode node) {
        for (AnnotationChangeListener l : listeners) {
            l.annotationChange(event, node);
        }
    }
    
    public interface AnnotationChangeListener {

        public void annotationChange(String event, DefaultMutableTreeNode node);
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        String event = e.getActionCommand();

        DefaultMutableTreeNode currentNode =
                (DefaultMutableTreeNode) annotationTree.getSelectionPath()
                        .getLastPathComponent();

        if (currentNode.isRoot()) {
            return;
        }
        
       notifyAnnotationListeners(event, currentNode);


    }

    public void clearTree() {
        rootNode = new DefaultMutableTreeNode("Annotation");

        annotationTree = new JTree(rootNode);

        scrollPane.setViewportView(annotationTree);
        
    }

}
