package de.unibi.airobots.bonsai.xcf.configuration;

import de.unibi.airobots.bonsai.core.Sensor;
import de.unibi.airobots.bonsai.core.CoreObjectFactory.SensorToConfigure;
import de.unibi.airobots.bonsai.xcf.sensors.XcfBtlMemorySensor;
import de.unibi.airobots.btl.data.Type;

/**
 * Configuration for {@link XcfBtlMemorySensor}.
 * <p>
 * Available options:
 * <ul>
 * <li>{@link ConfiguredXcfSensor#OPTION_PUBLISHER_NAME} (required): name of the
 * publisher to register on</li>
 * </ul>
 * 
 * @author lziegler
 * @param <T>
 *            sensor data type
 */
public class ConfiguredXcfBtlMemorySensor<T extends Type> extends
        ConfiguredXcfMemorySensor<T> {

    /**
     * {@inheritDoc}
     */
    @Override
    public Sensor<T> createInstance() {
        return new XcfBtlMemorySensor<T>(getDataType(), getMemoryName());
    }

    /**
     * Checks if the sensor that can be produced is suitable for the requested
     * configuration.
     * 
     * @param sensor
     *            requested configuration
     * @return <code>true</code> if suitable
     */
    public static boolean isSuitableFor(SensorToConfigure sensor) {
        return sensor.getSensorClass().equals(XcfBtlMemorySensor.class)
                && Type.class.isAssignableFrom(sensor.getDataTypeClass());
    }

}
